# -*- coding: utf-8 -*-


import sys


if sys.version_info[0] == 2:
    from .py2 import raise_, raise_from
else:
    from .py3 import raise_, raise_from


def reraise():
    """
    Does the same that ``raise`` without arguments do in Python2.
    But in both Python 2 and Python 3 (>= 3.3).
    """
    raise_(*sys.exc_info())


# silence pyflakes
assert reraise
assert raise_
assert raise_from



# -*- coding: utf-8 -*-


from .utils import prepare_raise


# Precompiled code for raise_from function and exec clause.
PRECOMPILED_RAISE = compile("raise error from cause", __file__, "exec")


# noinspection PyUnusedLocal
@prepare_raise
def raise_(type_, value=None, traceback=None):  # pylint: disable=W0613
    """
    Does the same as ordinary ``raise`` with arguments do in Python 2.
    But works in Python 3 (>= 3.3) also!

    Please checkout README on https://github.com/9seconds/pep3134
    to get an idea about possible pitfals. But short story is: please
    be pretty carefull with tracebacks. If it is possible, use sys.exc_info
    instead. But in most cases it will work as you expect.
    """

    if type_.__traceback__ is not traceback:
        raise type_.with_traceback(traceback)
    raise type_


def raise_from(error, cause):  # pylint: disable=W0613
    """
    Does the same as ``raise LALALA from BLABLABLA`` does in Python 3.
    But works in Python 2 also!

    Please checkout README on https://github.com/9seconds/pep3134
    to get an idea about possible pitfals. But short story is: please
    be pretty carefull with tracebacks. If it is possible, use sys.exc_info
    instead. But in most cases it will work as you expect.
    """

    # Okay, this is a story on Python packaging. First, there were
    # just a simple generic version
    #
    # raise error from cause
    #
    # and it was good. It worked on my machine but for some reason
    # during the package installation within Python 2 this file
    # was parsed and of curse blowed up on syntax error. So I need
    # to implement this line but in a dynamic way

    exec(PRECOMPILED_RAISE, globals(), locals())  # pylint: disable=W0122



# -*- coding: utf-8 -*-


import copy
import sys
import functools


def construct_exc_class(cls):
    """Constructs proxy class for the exception."""

    class ProxyException(cls, BaseException):
        __pep3134__ = True

        @property
        def __traceback__(self):
            if self.__fixed_traceback__:
                return self.__fixed_traceback__

            current_exc, current_tb = sys.exc_info()[1:]
            if current_exc is self:
                return current_tb

        def __init__(self, instance=None):
            super(ProxyException, self).__init__()

            self.__original_exception__ = instance
            self.__fixed_traceback__ = None

        def __getattribute__(self, item):
            if item == "__class__":
                return type(self.__original_exception__)
            return super(ProxyException, self).__getattribute__(item)

        def __repr__(self):
            return repr(self.__original_exception__)

        def with_traceback(self, traceback):
            instance = copy.copy(self)
            instance.__fixed_traceback__ = traceback
            return instance

    ProxyException.__name__ = cls.__name__

    return ProxyException


def prepare_raise(func):
    """
    Just a short decorator which shrinks
    full ``raise (E, V, T)`` form into proper ``raise E(V), T``.
    """

    @functools.wraps(func)
    def decorator(type_, value=None, traceback=None):
        if value is not None and isinstance(type_, Exception):
            raise TypeError("instance exception may not have a separate value")

        if value is None:
            if isinstance(type_, Exception):
                error = type_
            else:
                error = type_()
        else:
            error = type_(value)
        func(error, value, traceback)

    return decorator



# -*- coding: utf-8 -*-


import sys

from .utils import prepare_raise, construct_exc_class


# noinspection PyUnusedLocal
@prepare_raise
def raise_(type_, value=None, traceback=None):  # pylint: disable=W0613
    """
    Does the same as ordinary ``raise`` with arguments do in Python 2.
    But works in Python 3 (>= 3.3) also!

    Please checkout README on https://github.com/9seconds/pep3134
    to get an idea about possible pitfals. But short story is: please
    be pretty carefull with tracebacks. If it is possible, use sys.exc_info
    instead. But in most cases it will work as you expect.
    """

    prev_exc, prev_tb = sys.exc_info()[1:]
    proxy_class = construct_exc_class(type(type_))

    err = proxy_class(type_)
    err.__cause__ = None
    err.__suppress_context__ = False

    if getattr(prev_exc, "__pep3134__", False):
        prev_exc = prev_exc.with_traceback(prev_tb)
    err.__context__ = prev_exc

    if traceback:
        raise err.with_traceback(traceback), None, traceback
    else:
        raise err


def raise_from(exc, cause):
    """
    Does the same as ``raise LALALA from BLABLABLA`` does in Python 3.
    But works in Python 2 also!

    Please checkout README on https://github.com/9seconds/pep3134
    to get an idea about possible pitfals. But short story is: please
    be pretty carefull with tracebacks. If it is possible, use sys.exc_info
    instead. But in most cases it will work as you expect.
    """

    context_tb = sys.exc_info()[2]

    incorrect_cause = not (
        (isinstance(cause, type) and issubclass(cause, Exception)) or
        isinstance(cause, BaseException) or
        cause is None
    )
    if incorrect_cause:
        raise TypeError("exception causes must derive from BaseException")

    if cause is not None:
        if not getattr(cause, "__pep3134__", False):
            # noinspection PyBroadException
            try:
                raise_(cause)
            except:  # noqa pylint: disable=W0702
                cause = sys.exc_info()[1]
        cause.__fixed_traceback__ = context_tb

    # noinspection PyBroadException
    try:
        raise_(exc)
    except:  # noqa pylint: disable=W0702
        exc = sys.exc_info()[1]

    exc.__suppress_context__ = True
    exc.__cause__ = cause
    exc.__context__ = None

    raise exc



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from setuptools import setup, find_packages
from setuptools.command.test import test


REQUIREMENTS = []


with open("README.rst", "r") as resource:
    LONG_DESCRIPTION = resource.read()


# copypasted from http://pytest.org/latest/goodpractises.html
class PyTest(test):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        test.initialize_options(self)
        self.pytest_args = None  # pylint: disable=W0201

    def finalize_options(self):
        test.finalize_options(self)
        self.test_args = []  # pylint: disable=W0201
        self.test_suite = True  # pylint: disable=W0201

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        import sys
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name="pep3134",
    description="Backport of PEP 3134 (with PEP 415 and PEP 409) "
                "to Python 2 as close as possible",
    long_description="",
    version="0.1.2",
    author="Sergey Arkhipov",
    license="MIT",
    author_email="serge@aerialsounds.org",
    maintainer="Sergey Arkhipov",
    maintainer_email="serge@aerialsounds.org",
    url="https://github.com/9seconds/pep3134/",
    install_requires=REQUIREMENTS,
    tests_require=["pytest==2.6.1"],
    packages=find_packages(),
    cmdclass={'test': PyTest},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    zip_safe=True
)
